@@ -197,7 +197,7 @@ impl<'a> io::Read for Reader<'a> {
     /// You may learn the number of bytes available at any time by inspecting
     /// the return of [`Connection::process_new_packets`].
     #[cfg(read_buf)]
-    fn read_buf(&mut self, mut cursor: io::BorrowedCursor<'_>) -> io::Result<()> {
+    fn read_buf(&mut self, mut cursor: core::io::BorrowedCursor<'_>) -> io::Result<()> {
         let before = cursor.written();
         self.received_plaintext
             .read_buf(cursor.reborrow())?;
@@ -324,8 +324,8 @@ impl<Data> ConnectionCommon<Data> {
         let common = &mut self.core.common_state;
         Reader {
             received_plaintext: &mut common.received_plaintext,
-            /// Are we done? i.e., have we processed all received messages, and received a
-            /// close_notify to indicate that no new messages will arrive?
+            // Are we done? i.e., have we processed all received messages, and received a
+            // close_notify to indicate that no new messages will arrive?
             peer_cleanly_closed: common.has_received_close_notify
                 && !self.core.message_deframer.has_pending(),
             has_seen_eof: common.has_seen_eof,
@@ -371,15 +371,17 @@ impl<Data> ConnectionCommon<Data> {
         Self: Sized,
         T: io::Read + io::Write,
     {
-        let until_handshaked = self.is_handshaking();
         let mut eof = false;
         let mut wrlen = 0;
         let mut rdlen = 0;
 
         loop {
+            let until_handshaked = self.is_handshaking();
+
             while self.wants_write() {
                 wrlen += self.write_tls(io)?;
             }
+            io.flush()?;
 
             if !until_handshaked && wrlen > 0 {
                 return Ok((rdlen, wrlen));
@@ -410,11 +412,19 @@ impl<Data> ConnectionCommon<Data> {
                     // try a last-gasp write -- but don't predate the primary
                     // error.
                     let _ignored = self.write_tls(io);
+                    let _ignored = io.flush();
 
                     return Err(io::Error::new(io::ErrorKind::InvalidData, e));
                 }
             };
 
+            // if we're doing IO until handshaked, and we believe we've finished handshaking,
+            // but process_new_packets() has queued TLS data to send, loop around again to write
+            // the queued messages.
+            if until_handshaked && !self.is_handshaking() && self.wants_write() {
+                continue;
+            }
+
             match (eof, until_handshaked, self.is_handshaking()) {
                 (_, true, false) => return Ok((rdlen, wrlen)),
                 (_, false, _) => return Ok((rdlen, wrlen)),
@@ -435,10 +445,7 @@ impl<Data> ConnectionCommon<Data> {
             .map(Message::try_from)
         {
             Some(Ok(msg)) => Ok(Some(msg)),
-            Some(Err(err)) => {
-                self.send_fatal_alert(AlertDescription::DecodeError);
-                Err(err)
-            }
+            Some(Err(err)) => Err(self.send_fatal_alert(AlertDescription::DecodeError, err)),
             None => Ok(None),
         }
     }
@@ -661,23 +668,19 @@ impl<Data> ConnectionCore<Data> {
                     self.common_state.quic.alert = Some(AlertDescription::DecodeError);
                 }
 
-                if !self.common_state.is_quic() {
+                Err(if !self.common_state.is_quic() {
                     self.common_state
-                        .send_fatal_alert(AlertDescription::DecodeError);
-                }
-
-                Err(err)
-            }
-            Err(Error::PeerSentOversizedRecord) => {
-                self.common_state
-                    .send_fatal_alert(AlertDescription::RecordOverflow);
-                Err(Error::PeerSentOversizedRecord)
-            }
-            Err(Error::DecryptError) => {
-                self.common_state
-                    .send_fatal_alert(AlertDescription::BadRecordMac);
-                Err(Error::DecryptError)
+                        .send_fatal_alert(AlertDescription::DecodeError, err)
+                } else {
+                    err
+                })
             }
+            Err(err @ Error::PeerSentOversizedRecord) => Err(self
+                .common_state
+                .send_fatal_alert(AlertDescription::RecordOverflow, err)),
+            Err(err @ Error::DecryptError) => Err(self
+                .common_state
+                .send_fatal_alert(AlertDescription::BadRecordMac, err)),
             Err(e) => Err(e),
         }
     }
@@ -700,9 +703,8 @@ impl<Data> ConnectionCore<Data> {
                 // "An implementation which receives any other change_cipher_spec value or
                 //  which receives a protected change_cipher_spec record MUST abort the
                 //  handshake with an "unexpected_message" alert."
-                self.common_state
-                    .send_fatal_alert(AlertDescription::UnexpectedMessage);
-                return Err(Error::PeerMisbehaved(
+                return Err(self.common_state.send_fatal_alert(
+                    AlertDescription::UnexpectedMessage,
                     PeerMisbehaved::IllegalMiddleboxChangeCipherSpec,
                 ));
             } else {
@@ -716,9 +718,9 @@ impl<Data> ConnectionCore<Data> {
         let msg = match Message::try_from(msg) {
             Ok(msg) => msg,
             Err(err) => {
-                self.common_state
-                    .send_fatal_alert(AlertDescription::DecodeError);
-                return Err(err);
+                return Err(self
+                    .common_state
+                    .send_fatal_alert(AlertDescription::DecodeError, err));
             }
         };
 
