@@ -1,8 +1,12 @@
+use std::fmt;
+
+use crate::error::InvalidMessage;
 use crate::key;
 use crate::msgs::codec;
 use crate::msgs::codec::{Codec, Reader};
+
 /// An externally length'd payload
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Clone, Eq, PartialEq)]
 pub struct Payload(pub Vec<u8>);
 
 impl Codec for Payload {
@@ -10,8 +14,8 @@ impl Codec for Payload {
         bytes.extend_from_slice(&self.0);
     }
 
-    fn read(r: &mut Reader) -> Option<Self> {
-        Some(Self::read(r))
+    fn read(r: &mut Reader) -> Result<Self, InvalidMessage> {
+        Ok(Self::read(r))
     }
 }
 
@@ -35,16 +39,22 @@ impl Codec for key::Certificate {
         bytes.extend_from_slice(&self.0);
     }
 
-    fn read(r: &mut Reader) -> Option<Self> {
+    fn read(r: &mut Reader) -> Result<Self, InvalidMessage> {
         let len = codec::u24::read(r)?.0 as usize;
         let mut sub = r.sub(len)?;
         let body = sub.rest().to_vec();
-        Some(Self(body))
+        Ok(Self(body))
+    }
+}
+
+impl fmt::Debug for Payload {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        hex(f, &self.0)
     }
 }
 
 /// An arbitrary, unknown-content, u24-length-prefixed payload
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Clone, Eq, PartialEq)]
 pub struct PayloadU24(pub Vec<u8>);
 
 impl PayloadU24 {
@@ -59,16 +69,22 @@ impl Codec for PayloadU24 {
         bytes.extend_from_slice(&self.0);
     }
 
-    fn read(r: &mut Reader) -> Option<Self> {
+    fn read(r: &mut Reader) -> Result<Self, InvalidMessage> {
         let len = codec::u24::read(r)?.0 as usize;
         let mut sub = r.sub(len)?;
         let body = sub.rest().to_vec();
-        Some(Self(body))
+        Ok(Self(body))
+    }
+}
+
+impl fmt::Debug for PayloadU24 {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        hex(f, &self.0)
     }
 }
 
 /// An arbitrary, unknown-content, u16-length-prefixed payload
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Clone, Eq, PartialEq)]
 pub struct PayloadU16(pub Vec<u8>);
 
 impl PayloadU16 {
@@ -91,16 +107,22 @@ impl Codec for PayloadU16 {
         Self::encode_slice(&self.0, bytes);
     }
 
-    fn read(r: &mut Reader) -> Option<Self> {
+    fn read(r: &mut Reader) -> Result<Self, InvalidMessage> {
         let len = u16::read(r)? as usize;
         let mut sub = r.sub(len)?;
         let body = sub.rest().to_vec();
-        Some(Self(body))
+        Ok(Self(body))
+    }
+}
+
+impl fmt::Debug for PayloadU16 {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        hex(f, &self.0)
     }
 }
 
 /// An arbitrary, unknown-content, u8-length-prefixed payload
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Clone, Eq, PartialEq)]
 pub struct PayloadU8(pub Vec<u8>);
 
 impl PayloadU8 {
@@ -123,10 +145,27 @@ impl Codec for PayloadU8 {
         bytes.extend_from_slice(&self.0);
     }
 
-    fn read(r: &mut Reader) -> Option<Self> {
+    fn read(r: &mut Reader) -> Result<Self, InvalidMessage> {
         let len = u8::read(r)? as usize;
         let mut sub = r.sub(len)?;
         let body = sub.rest().to_vec();
-        Some(Self(body))
+        Ok(Self(body))
     }
 }
+
+impl fmt::Debug for PayloadU8 {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        hex(f, &self.0)
+    }
+}
+
+// Format an iterator of u8 into a hex string
+pub(super) fn hex<'a>(
+    f: &mut fmt::Formatter<'_>,
+    payload: impl IntoIterator<Item = &'a u8>,
+) -> fmt::Result {
+    for b in payload {
+        write!(f, "{:02x}", b)?;
+    }
+    Ok(())
+}
